# -*- coding: utf-8 -*-
"""
Created on Wed Jan 22 10:07:12 2025

@author: mstep
"""

#%%
import math
import tkinter as tk
from PIL import Image, ImageDraw, ImageTk

class SierpinskiGasketApp:
    def __init__(self):
        # Drawing parameters.
        margin = 10
        self.drawing = False
        x = 200
        y = x * math.sqrt(3)
        self.corners = [
            (x + margin, margin),
            (margin, y + margin),
            (2 * x + margin, y + margin),
        ]
        self.canvas_wid = round(2 * x + 2 * margin)
        self.canvas_hgt = round(y + 2 * margin)

        self.window = tk.Tk()
        self.window.title('sierpinski_gasket')
        self.window.protocol('WM_DELETE_WINDOW', self.kill_callback)
        self.window.geometry('300x400')

        # Drawing parameters.
        wid = 300
        hgt = 300
        margin = 10

        # Parameters.
        frame = tk.Frame(self.window)
        frame.pack(side=tk.TOP, padx=margin, pady=(margin,0),
            expand=False, fill=tk.X)
        label = tk.Label(frame, text='Depth:')
        label.pack(side=tk.LEFT)
        self.depth_var = tk.StringVar(value='2')
        self.depth_spinbox = tk.Spinbox(frame, from_=0, to_=7, increment=1,
            justify=tk.RIGHT, width=3, textvariable=self.depth_var,
            state='readonly')
        self.depth_spinbox.pack(side=tk.LEFT)
        button = tk.Button(frame, text='Draw', width=6,
            command=self.draw_gasket)
        button.pack(side=tk.LEFT, padx=20)

        # Canvas for drawing the gasket.
        self.canvas = tk.Canvas(self.window, bg='white',
            width=wid, height=hgt, borderwidth=0, highlightthickness=0)
        self.canvas.pack(side=tk.TOP, anchor=tk.NW,
            padx=margin, pady=(5,margin),
            expand=True, fill=tk.BOTH)

        self.window.bind('<Return>', lambda _: self.draw_gasket())

        # Size the window to fit.
        self.canvas.update()
        wid = self.canvas_wid + 2 * margin
        y = self.canvas.winfo_y()
        hgt = self.canvas_hgt + 2 * margin + y
        self.window.geometry(f'{wid}x{hgt}')
        self.window.resizable(False, False)

        # Display the window.
        self.depth_spinbox.focus_force()
        self.window.mainloop()

    def kill_callback(self):
        # Destroy the tkinter window.
        self.window.destroy()

    def draw_gasket(self):
        '''Draw the gasket.'''
        # Get parameters.
        depth = int(self.depth_var.get())

        # Generate the gasket.
        gasket = self.make_gasket(depth, self.canvas_wid, self.canvas_hgt,
            self.corners, 'white', 'orange', 'lightgreen', 'black')

        # Display the gasket.
        self.canvas.delete(tk.ALL)
        self.gasket_photo_image = ImageTk.PhotoImage(gasket)
        cx = self.canvas.winfo_width() / 2
        cy = self.canvas.winfo_height() / 2
        self.canvas.create_image(cx, cy, image=self.gasket_photo_image)

        # Save the gasket.
        gasket.save('gasket.png')

    def make_gasket(self, depth, wid, hgt, corners, outside_color,
            fill, background, outline):
        '''Make the gasket image.'''
        # Make the image, initially filled with outside_color.
        image = Image.new('RGB', (wid, hgt), outside_color)
        dr = ImageDraw.Draw(image)

        # Fill the triangle with the fill color.
        dr.polygon(corners, fill=fill, outline=outline)

        # Recursively draw the gasket.
        self.remove_center(depth, dr, corners, background, outline)
        return image

    def remove_center(self, depth, dr, corners, background, outline):
        '''Clear the center of this triangle.'''
        # Find points p1, p2, and p3.
        p0, p1, p2 = corners
        x3 = (p0[0] + p1[0]) / 2
        y3 = (p0[1] + p1[1]) / 2
        p3 = (x3, y3)
        x4 = (p0[0] + p2[0]) / 2
        y4 = (p0[1] + p2[1]) / 2
        p4 = (x4, y4)
        x5 = (p1[0] + p2[0]) / 2
        y5 = (p1[1] + p2[1]) / 2
        p5 = (x5, y5)

        # Fill the inner triangle with the background color.
        dr.polygon((p3, p4, p5), fill=background, outline=outline)

        # Recursively draw the corner triangles.
        depth -= 1
        if depth >= 0:
            self.remove_center(depth, dr, (p0, p3, p4), background, outline)
            self.remove_center(depth, dr, (p3, p1, p5), background, outline)
            self.remove_center(depth, dr, (p4, p5, p2), background, outline)

#%%
SierpinskiGasketApp()
print('Done')
